// This work is licensed under the Creative Commons Attribution-ShareAlike 4.0
// International Public License.
// Visit https://creativecommons.org/licenses/by-sa/4.0/.
// SPDX-License-Identifier: CC-BY-SA-4.0
//     yuqk f_path_00.pov +w1200 +h900  (quick and ugly)
//     yuqk f_path_00.pov +w1200 +h900 +a0.05 +am2 +r3 +j3.33 +d +p
#version unofficial 3.8; // yuqk
#if (file_exists("version.inc"))
    #include "version.inc"
#end
#if (!defined(Fork_yuqk))
    #error "This POV-Ray SDL code requires the yuqk fork."
#end
global_settings { assumed_gamma 1 }
#declare Grey50 = srgb <0.5,0.5,0.5>;
background { Grey50 }
#declare Camera00 = camera {
    perspective
    location <4.9,4.9,-5.901>
    sky y
    angle 35
    right x*(image_width/image_height)
    look_at <0,0,0>
}
#declare VarOrthoMult =
    2.0/max(image_width/image_height,image_height/image_width);
#declare Camera01z = camera {
    orthographic
    location <0,0,-2>
    direction z
    right VarOrthoMult*x*max(1,image_width/image_height)
    up VarOrthoMult*y*max(1,image_height/image_width)
}
#declare White = srgb <1,1,1>;
#declare Light00 = light_source { <50,150,-250>, White }
#declare Red = srgb <1,0,0>;
#declare CylinderX = cylinder { -1*x, 1*x, 0.01 pigment { Red } }
#declare Green = srgb <0,1,0>;
#declare CylinderY = cylinder { -1*y, 1*y, 0.01 pigment { Green } }
#declare Blue = srgb <0,0,1>;
#declare CylinderZ = cylinder { -1*z, 1*z, 0.01 pigment { Blue } }
#declare AmericanBlue = srgb <0.23137,0.23137,0.42745>;
#declare AmericanBronze = srgb <0.22353,0.09412,0.00784>;
#declare AmericanBrown = srgb <0.50196,0.25098,0.25098>;
#declare AmericanGold = srgb <0.82745,0.68627,0.21569>;
#declare AmericanGreen = srgb <0.20392,0.70196,0.20392>;
#declare AmericanOrange = srgb <1,0.5451,0>;
#declare AmericanPink = srgb <1,0.59608,0.6>;
#declare AmericanPurple = srgb <0.26275,0.1098,0.32549>;
#declare AmericanRed = srgb <0.70196,0.12941,0.20392>;
#declare AmericanRose = srgb <1,0.01176,0.24314>;
#declare AmericanViolet = srgb <0.33333,0.10588,0.54902>;
#declare AmericanYellow = srgb <0.94902,0.70588,0>;
#declare Amazonite = srgb <0,0.76863,0.6902>;
#declare Americano = srgb <0.52941,0.45882,0.43137>;
#declare ColorMap14 = color_map {
    [ -1 AmericanBlue ]
    [  0 AmericanBlue ]
    [  1 AmericanBronze ]
    [  2 AmericanBrown ]
    [  3 AmericanGold ]
    [  4 AmericanGreen ]
    [  5 AmericanOrange ]
    [  6 AmericanPink ]
    [  7 AmericanPurple ]
    [  8 AmericanRed ]
    [  9 AmericanRose ]
    [ 10 AmericanViolet ]
    [ 11 AmericanYellow ]
    [ 12 Amazonite ]
    [ 13 Americano ]
}
// Array of points / vertices used to set up all f_path functions
#declare VertexAry = array[13][3] {
  {+0.0,+0.0,+0.0},
  {+0.5,+0.0,+0.0},
  {+0.5,+0.5,+0.0},
  {-0.5,+0.5,+0.0},
  {-0.5,-0.5,+0.0},
  {+1.0,-0.5,+0.0},
  {+1.0,+1.0,+0.0},
  {-1.0,+1.0,+0.0},
  {-1.0,-1.0,+0.0},
  {+1.5,-1.0,+0.0},
  {+1.5,+1.5,+0.0},
  {-1.5,+1.5,+0.0},
  {-1.5,-1.5,+0.0}
}
#include "functions.inc"
// Function f_enc16x_ui4 defined in functions.inc
// Function f_path defined in functions.inc

// === 3x3 doubles (64bit floats)
#declare Cnct_0_dbl =
    f_enc16x_ui4(
    0,1,   // Connect point 0 to 1
    13,0,  // Set to taper connection to 0/15
    1,2,   // Connect point 1 to 2
    14,7,  // Set negative region to 7/15*offset (taper above active)
    2,0,   // Connect 2 to 0 (could reverse with 0,2)
    15,15  // End of connections.
    0,0, 0,0
);
#declare FnDbls = function (x,y,z) {
    f_path(x,y,z,0,0,0.05,
        VertexAry[0][0],VertexAry[0][1],VertexAry[0][2],
        VertexAry[1][0],VertexAry[1][1],VertexAry[1][2],
        VertexAry[2][0],VertexAry[2][1],VertexAry[2][2],
        Cnct_0_dbl, 0.0, 0.0, 0.0
    )
}
#declare IsoDbls = isosurface {
    function { FnDbls(x,y,z) }
    contained_by { box { <-2,-2,-0.2>,<2,2,0.2> } }
    threshold 0
    accuracy 0.0005
    max_gradient 1.1
    pigment { AmericanRose }
}

// === 6x3 singles (32 bit floats)
#declare VertexAryF32 = array[9] {
  f_enc2x_f32(VertexAry[0][0],VertexAry[0][1]),
  f_enc2x_f32(VertexAry[0][2],VertexAry[1][0]),
  f_enc2x_f32(VertexAry[1][1],VertexAry[1][2]),
  f_enc2x_f32(VertexAry[2][0],VertexAry[2][1]),
  f_enc2x_f32(VertexAry[2][2],VertexAry[3][0]),
  f_enc2x_f32(VertexAry[3][1],VertexAry[3][2]),
  f_enc2x_f32(VertexAry[4][0],VertexAry[4][1]),
  f_enc2x_f32(VertexAry[4][2],VertexAry[5][0]),
  f_enc2x_f32(VertexAry[5][1],VertexAry[5][2])
}
#declare Cnct_0_sngl =
    f_enc16x_ui4(
    0,1, 1,2, 2,3,
    3,4, 4,5, 5,0,
    15,15, 0,0
);
#declare FnSngls = function (x,y,z) {
    f_path(x,y,z,1,0,0.05,
        VertexAryF32[0],VertexAryF32[1],VertexAryF32[2],
        VertexAryF32[3],VertexAryF32[4],VertexAryF32[5],
        VertexAryF32[6],VertexAryF32[7],VertexAryF32[8],
        Cnct_0_sngl, 0.0, 0.0, 0.0
    )
}
#declare IsoSngls = isosurface {
    function { FnSngls(x,y,z) }
    contained_by { box { <-2,-2,-0.2>,<2,2,0.2> } }
    threshold 0
    accuracy 0.0005
    max_gradient 1.1
    pigment { AmericanRose }
}

// === 9x3 21 bit floats
#declare VertexAryF21 = array[9] {
  f_enc3x_f21(VertexAry[0][0],VertexAry[0][1],VertexAry[0][2],5),
  f_enc3x_f21(VertexAry[1][0],VertexAry[1][1],VertexAry[1][2],5),
  f_enc3x_f21(VertexAry[2][0],VertexAry[2][1],VertexAry[2][2],5),
  f_enc3x_f21(VertexAry[3][0],VertexAry[3][1],VertexAry[3][2],5),
  f_enc3x_f21(VertexAry[4][0],VertexAry[4][1],VertexAry[4][2],5),
  f_enc3x_f21(VertexAry[5][0],VertexAry[5][1],VertexAry[5][2],5),
  f_enc3x_f21(VertexAry[6][0],VertexAry[6][1],VertexAry[6][2],5),
  f_enc3x_f21(VertexAry[7][0],VertexAry[7][1],VertexAry[7][2],5),
  f_enc3x_f21(VertexAry[8][0],VertexAry[8][1],VertexAry[8][2],5)
}
#declare Cnct_0_f21 =
    f_enc16x_ui4(
    0,1, 1,2, 2,3,
    3,4, 4,5, 5,6,
    6,7, 7,8
);
#declare Cnct_1_f21 =
    f_enc16x_ui4(
    8,0, 15,15, 0,0,
    0,0, 0,0, 0,0,
    0,0, 0,0
);
#declare FnF21 = function (x,y,z) {
    f_path(x,y,z,9,0,0.05,
        VertexAryF21[0],VertexAryF21[1],VertexAryF21[2],
        VertexAryF21[3],VertexAryF21[4],VertexAryF21[5],
        VertexAryF21[6],VertexAryF21[7],VertexAryF21[8],
        Cnct_0_f21, Cnct_1_f21, 0.0, 0.0
    )
}
#declare IsoF21 = isosurface {
    function { FnF21(x,y,z) }
    contained_by { box { <-2,-2,-0.2>,<2,2,0.2> } }
    threshold 0
    accuracy 0.0005
    max_gradient 1.1
    pigment { AmericanRose }
}

// === 12x2 21 bit floats
#declare VertexAryF21_2 = array[9] {
  f_enc3x_f21(VertexAry[0][0],VertexAry[0][1],VertexAry[1][0],5),
  f_enc3x_f21(VertexAry[1][1],VertexAry[2][0],VertexAry[2][1],5),
  f_enc3x_f21(VertexAry[3][0],VertexAry[3][1],VertexAry[4][0],5),
  f_enc3x_f21(VertexAry[4][1],VertexAry[5][0],VertexAry[5][1],5),
  f_enc3x_f21(VertexAry[6][0],VertexAry[6][1],VertexAry[7][0],5),
  f_enc3x_f21(VertexAry[7][1],VertexAry[8][0],VertexAry[8][1],5),
  f_enc3x_f21(VertexAry[9][0],VertexAry[9][1],VertexAry[10][0],5),
  f_enc3x_f21(VertexAry[10][1],VertexAry[11][0],VertexAry[11][1],5),
  f_enc3x_f21(VertexAry[12][0],VertexAry[12][1],0.0,5)
}
#declare Cnct_0_f21_2 =
    f_enc16x_ui4(
    0,1, 1,2, 2,3,
    3,4, 4,5, 5,6,
    6,7, 7,8
);
#declare Cnct_1_f21_2 =
    f_enc16x_ui4(
    8,9, 9,10, 10,11,
    11,12, 15,15, 0,0,
    0,0, 0,0
);
#declare FnF21_2 = function (x,y,z) {
    f_path(x,y,z,13,0,0.07,
        VertexAryF21_2[0],VertexAryF21_2[1],VertexAryF21_2[2],
        VertexAryF21_2[3],VertexAryF21_2[4],VertexAryF21_2[5],
        VertexAryF21_2[6],VertexAryF21_2[7],VertexAryF21_2[8],
        Cnct_0_f21_2, Cnct_1_f21_2, 0.0, 0.0
    )
}
#declare IsoF21_2 = isosurface {
    function { FnF21_2(x,y,z) }
    contained_by { box { <-2,-2,-0.2>,<2,2,0.2> } }
    threshold 0
    accuracy 0.0005
    max_gradient 1.1
}

// === Set up some pigments using return modes of f_path
#declare FnF21_2p2 = function (x,y,z) {
    f_path(x,y,z,13,2,0.07,
        VertexAryF21_2[0],VertexAryF21_2[1],VertexAryF21_2[2],
        VertexAryF21_2[3],VertexAryF21_2[4],VertexAryF21_2[5],
        VertexAryF21_2[6],VertexAryF21_2[7],VertexAryF21_2[8],
        Cnct_0_f21_2, Cnct_1_f21_2, 0.0, 0.0
    )
}
#declare FnF21_2p3 = function (x,y,z) {
    f_path(x,y,z,13,3,0.07,
        VertexAryF21_2[0],VertexAryF21_2[1],VertexAryF21_2[2],
        VertexAryF21_2[3],VertexAryF21_2[4],VertexAryF21_2[5],
        VertexAryF21_2[6],VertexAryF21_2[7],VertexAryF21_2[8],
        Cnct_0_f21_2, Cnct_1_f21_2, 0.0, 0.0
    )
}
#declare Pig02 = pigment {
    function { FnF21_2p2(x,y,z) }
    raw_wave
    color_map { ColorMap14 }
}
#declare Pig03 = pigment {
    function { FnF21_2p3(x,y,z) }
    raw_wave
    color_map { ColorMap14 }
}
#declare Fin00 = finish { phong 0.5 }
#declare Txtr02 = texture { pigment { Pig02 } finish { Fin00 } }
#declare Txtr03 = texture { pigment { Pig03 } finish { Fin00 } }
#declare Obj02 = object { IsoF21_2 texture { Txtr02 } }
#declare Obj03 = object { IsoF21_2 texture { Txtr03 } }

//--- scene ---
    camera { Camera00 }
    light_source { Light00 }
//  camera { Camera01z }
    object { CylinderX }
    object { CylinderY }
    object { CylinderZ }
//  object { IsoDbls }
//  object { IsoSngls }
//  object { IsoF21 }
    object { Obj02 }
//  object { Obj03 }
